/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.client;

import org.oasis.wsrf.lifetime.ImmediateResourceTermination;
import org.oasis.wsrf.lifetime.WSResourceLifetimeServiceAddressingLocator;

import java.net.URL;
import javax.xml.rpc.Stub;
import javax.xml.rpc.ServiceException;

import org.globus.tutorial.filestore.FileService;
import org.globus.tutorial.filestore.service.FileServiceAddressingLocator;
import org.globus.tutorial.filestore.service.FilestoreFactoryServiceAddressingLocator;
import org.globus.tutorial.filestore.FileStoreFactory;
import org.globus.tutorial.types.filestore.FileStoreException;

import org.globus.tutorial.types.filestore.Data;
import org.globus.tutorial.types.filestore.ArrayOfDouble;
import org.globus.tutorial.types.filestore.RequestFileType;
import org.globus.tutorial.types.filestore.ConcatInput;
import org.globus.tutorial.types.filestore.ArrayOfEPR;
import org.globus.tutorial.types.filestore.VoidType;
import org.globus.tutorial.types.filestore.VoidType1;
import org.globus.tutorial.types.filestore.RequestFileType;

import org.apache.axis.message.addressing.EndpointReference;
import org.apache.axis.message.addressing.EndpointReferenceType;

import org.globus.wsrf.impl.security.authentication.Constants;
import org.globus.wsrf.impl.security.authorization.NoAuthorization;


public class FileServiceUtil {

    private static final String PERSIST_FILESTORE_FACTORY 
        = "/wsrf/services/PersistentFileStoreFactoryService";
    private static final String PERSIST_FILESTORE_SERVICE 
        = "/wsrf/services/PersistentFileService";

    private static final String FILESTORE_FACTORY 
        = "/wsrf/services/FileStoreFactoryService";
    private static final String FILESTORE_SERVICE 
        = "/wsrf/services/FileService";
    
    public static EndpointReference createFile(int numBytes, 
                                               RequestFileType fileType, 
                                               int lifeTime, String hostPort)
        throws Exception {
        
        return createFile(numBytes, fileType, lifeTime, hostPort, false);
    }

    /**
     *  Method to create a file using the FileStoreFactory service. 
     *
     * @param numBytes
     *        Requested size of file.
     * @param fileType
     *        Type of file to indicate whether 1D, 2D or 3D needs to
     *        be stored.
     * @param lifeTime
     *        Time to live for the file(resource)
     * @param hostPort
     *        String of format host:port, host and port on which the
     *        FileStore service is running.
     */
    public static EndpointReference createFile(int numBytes, 
                                               RequestFileType fileType, 
                                               int lifeTime, String hostPort,
                                               boolean persist)
        throws Exception {
        
        String err = "Error creating file\n";

        FileStoreFactory factory = null;
	EndpointReferenceType ref = null;

        // (ADD 1) Construct URL  to contact the
        // FileStoreFactoryService (FSFS)

        // Checking that parameters passed to this method are valid.
        if(numBytes < 1 || lifeTime < 1 ) {
            throw new Exception("Error in input parameters for file creation");
        }
        
        // (ADD 2) Construct Request for 
        // parameter: file size in bytes, type of file (1D), lifetime of
        // file resource


        // (ADD 3) Set up FileStoreFactory stub to invoke create File method on
        // it. Assign the created FileStoreFactory stub to "factory"
        // variable
        // factory =

        // sets up security properties. The FSFS requires security
        // This makes the client use GSI Secure Message security, with
        // integrity only (i.e message is just signed)
        ((Stub)factory)._setProperty(Constants.GSI_SEC_MSG, 
                                     Constants.SIGNATURE);
        // This disables any authorization of service.
        ((Stub)factory)._setProperty(Constants.AUTHORIZATION, 
                                     NoAuthorization.getInstance());

        // (ADD 4) Invoke create file method and store returned
        // endpoint reference type object to variablae "ref"
        // ref =

        // return endpoint reference.
	return new EndpointReference(ref);
    }

    public static void concatenate(EndpointReference resultFile,
                                   EndpointReference[] eprs,
                                   String hostPort) throws Exception {
        concatenate(resultFile, eprs, hostPort, false);
        
    }

    /**
     * Method to use the FileService to concatenate a set of files of
     * same type.
     *
     * @param eprs
     *         Array of EPRs that point to files that need to be
     *         concatenated.
     * @param hostPort
     *        String of format host:port, host and port on which the
     *        FileStore service is running.
     */
    public static void concatenate(EndpointReference resultFile,
                                   EndpointReference[] eprs,
                                   String hostPort, boolean persist)
        throws Exception {

        String err = "Error concatenating files containing 1D data\n";

        // (ADD 1) Construct URL to FileStore Service

        // (ADD 2) Create ConcatInput, parameter ArrayOfEPR and EPR to
        // write concatenated output to.

        // (ADD 3) Get stub to FileStore service

        // (ADD 4) Invoke concatenate.
    }

    public static Data fetchContents(EndpointReference epr) throws Exception {

        Data returnedData = null;

        // (ADD 1) Get stub to FileService using FileServiceAddressingLocator

        // (ADD 2) invoke fetch operation and assign returned data to
        // variable "returnedData"

        return returnedData;
    }

    public static void deleteFile(EndpointReference epr) 
        throws Exception {

        // (ADD 1) Get stub to ImmediateResourceTermination port type
        // using the WSResourceLifetimeServiceAddressingLocator.

        // (ADD 2) Invoke destroy
    }

    public static void storeContents(EndpointReference epr, Data data) 
        throws Exception {

        // (ADD 1) Get stub to FileService using
        // FileServiceAddressingLocator

        // (ADD 2) Invoke store operation
    }

    public static void printData(Data data) {
        
        if (data == null) {
            System.out.println("Data is null");
        }

        ArrayOfDouble[] array = data.getArray();
        if (array == null) {
            System.out.println("Data does not have any data");
        }

        int size = array.length;
        System.out.println("Data has " + size + " sets of data");
        
        for (int i=0; i < size; i++) {
            double[] value = array[i].getValue();
            System.out.println("\t Set " + i + " size: " + value.length);
            String val = "\t";
            for (int j=0; j < value.length; j++) {
                val = val + "\t" + value[j];
            }
            System.out.println(val);
        }
    }

    /**
     * Method to erase contents file
     *
     * @parameter epr
     *            Endpoint Reference to file that needs to be deleted.
     */
    public static void eraseContents(EndpointReference epr) 
        throws Exception {

        // (ADD 1) Get stub to FileService port using
        //  FileServiceAddressingLocator

        // (ADD 2) Construct parameter. Refer to Javadocs.

        // (ADD 3) Invoke erase contents methhod.
    }
}


