/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.client;

import org.apache.axis.message.addressing.EndpointReference;

import org.globus.tutorial.types.filestore.Data;
import org.globus.tutorial.types.filestore.RequestFileType;
import org.globus.tutorial.types.filestore.ArrayOfDouble;

public class GTAdvancedTutorial {

    public static void main(String[] args) throws Exception {

        if (args.length < 2) {
            System.out.println("Arguments: <host:port> <persistent>");
            System.out.println(" where <host:port> is hostname:portnumber");
            System.out.println(" <persistent> is true or false depending"
                               + " where persistent service is being accessed"
                               + " or not");
            System.out.println("E.g. gt4-advanced-tutorial localhost:8080 "
                               + " true");
            
            System.exit(-1);
        }

        boolean persist  = false;
        if (args[1].equals("true")) {
            persist = true;
        } 
        
        // create, store and fetch from files.        
        exercise1(args[0], persist);
        // create file, generate sequences, xproduct of sequence
        exercise2(args[0], persist);
    }

    /**
     * In this method operations to create different kinds of files,
     * store/retrieve data from them, concatenate files and delete files
     * are exercised. Operations createFile, storeContents,
     * fetchContents, concatenate and deleteFile from class
     * FileServiceUtil are used.
     */
    public static void exercise1(String hostPort, boolean persist)
        throws Exception {
        
        System.out.println("\n\nExercise 1");

        // FIXME: lifetime in seconds ?

        // Create a file to store 1D data of size 10 bytes and
        // lifetime of 100. Use createFile method in FileServiceUtil 
        EndpointReference epr1 = 
            FileServiceUtil.createFile(10, RequestFileType.value1, 100,
                                       hostPort, persist);

        // Sample 1D data
        double[] oneD = new double[] { 0.01, 0.04 };

        // Construct Data object using the above sample data to be
        //  able store using FileService
        ArrayOfDouble oneDArray = new ArrayOfDouble(oneD);
        ArrayOfDouble[] dataArrays = new ArrayOfDouble[] { oneDArray };
        Data oneDData = new Data(dataArrays);
        
        // Store the data using storeContents method in 
        // FileServiceUtil.
        FileServiceUtil.storeContents(epr1, oneDData);


        // Create a file to store 2D data of size 20 bytes and
        // lifetime of 100
        EndpointReference epr2 = 
            FileServiceUtil.createFile(20, RequestFileType.value2 , 100,
                                       hostPort, persist);

        // Sample 2D data
        ArrayOfDouble x = 
            new ArrayOfDouble(new double[] { 0.03, 0.04 });
        ArrayOfDouble y = 
            new ArrayOfDouble(new double[] { 0.23, 0.24 });

        // Construct Data object using above.
        ArrayOfDouble[] dataArrays2 = new ArrayOfDouble[] { x, y };
        Data twoDData = new Data(dataArrays2);

        // Store data 
        FileServiceUtil.storeContents(epr2, twoDData);

        // Create a file to store 3D data of size 30 bytes
        EndpointReference epr3 = 
            FileServiceUtil.createFile(30, RequestFileType.value3 ,
                                       100, hostPort, persist);
        
        // Store some 3D data
        ArrayOfDouble x3 = 
            new ArrayOfDouble(new double[] { 0.03, 0.04 });
        ArrayOfDouble y3 = 
            new ArrayOfDouble(new double[] { 0.23, 0.24 });
        ArrayOfDouble z3 = 
            new ArrayOfDouble(new double[] { 0.223, 0.224 });

        // Construct Data object using above.
        ArrayOfDouble[] dataArrays3 = new ArrayOfDouble[] { x3, y3, z3 };
        Data threeDData = new Data(dataArrays3);

        // Store data
        FileServiceUtil.storeContents(epr3, threeDData);

        // Get data from 1D file using fetchContents method in
        //  FileServiceUtil.
        Data output1 = FileServiceUtil.fetchContents(epr1);

        System.out.println("\nData from 1D file");
        FileServiceUtil.printData(output1);

        // Get data from 2D 
        Data output2 = FileServiceUtil.fetchContents(epr2);

        System.out.println("\nData from 2D file");
        FileServiceUtil.printData(output2);

        // Get data from 3D         
        Data output3 = FileServiceUtil.fetchContents(epr3);

        System.out.println("\nData from 3D file");
        FileServiceUtil.printData(output3);

        // Create another 3D file to concatenate.
        EndpointReference epr4 = 
            FileServiceUtil.createFile(30, RequestFileType.value3 ,
                                       100, hostPort, persist);

        // Store some 3D data
        x3 = new ArrayOfDouble(new double[] { 0.031, 0.041 });
        y3 = new ArrayOfDouble(new double[] { 0.231, 0.241 });
        z3 = new ArrayOfDouble(new double[] { 0.2231, 0.2241 });

        // Construct Data object for above.
        dataArrays3 = new ArrayOfDouble[] { x3, y3, z3 };
        threeDData = new Data(dataArrays3);

        // Store data
        FileServiceUtil.storeContents(epr4, threeDData);

        // Get data from second 3D file
        Data output4 = FileServiceUtil.fetchContents(epr4);

        System.out.println("\nData from second 3D file");
        FileServiceUtil.printData(output4);

        // Create another 3D file to store output of concatenation.
        EndpointReference concatFile = 
            FileServiceUtil.createFile(60, RequestFileType.value3 ,
                                       100, hostPort, persist);
        
        // Concatenate files using the concatenate method in
        // FileServiceUtil.
        FileServiceUtil.concatenate(concatFile, 
                                    new EndpointReference[] { epr3, epr4 }, 
                                    hostPort, persist);

        // Fetch data from concatenated file.
        Data concatenatedOutput = 
            FileServiceUtil.fetchContents(concatFile);

        System.out.println("\nData from concatenated file");
        FileServiceUtil.printData(concatenatedOutput);

        // Delete file epr1 (to prevent exceeding quota) using
        // deleteFile method in FileServiceUtil
        FileServiceUtil.deleteFile(epr1);

        // Try to fetch contents from epr1
        try {
            output1 = FileServiceUtil.fetchContents(epr1);
        } catch (Exception exp) {
            System.err.println("\nAccessing deleted file " + exp.getMessage());
        }

        // Delete all other files (2D, 3D, second 3D,
        // concatenated file).
        FileServiceUtil.deleteFile(epr2);
        FileServiceUtil.deleteFile(epr3);
        FileServiceUtil.deleteFile(epr4);
        FileServiceUtil.deleteFile(concatFile);
    }


    /**
     * In this method operations to create files and use EPR as
     * parameters to other services like the Sequence Generator service
     * and Xproduct Generator Service are exercised. For generator
     * services, code in GeneratorServicesUtil should be used.
     */
    public static void exercise2(String hostPort, boolean persist) 
        throws Exception {
        
        System.out.println("\n\nExercise 2");
        // create 1D file with size 10 bytes and lifetime of 100
        EndpointReference seq1Ref = 
            FileServiceUtil.createFile(10, RequestFileType.value1, 100,
                                       hostPort, persist);

        // invoke sequence generator to write 1D data to
        // file. generate sequence from 1, upto 5, with step as 2. Use
        // seqgen from GeneratorSrevicesUtil class.
        GeneratorServicesUtil.seqgen(1, 2, 5, seq1Ref, hostPort);

        // Fetch data from file to see sequence generator's output
        Data data1 = FileServiceUtil.fetchContents(seq1Ref);

        System.out.println("\nData from sequence generated for X"); 
        FileServiceUtil.printData(data1);

        // create 1D file with size 10 bytes and lifetime fo 100
        EndpointReference seq2Ref = 
            FileServiceUtil.createFile(10, RequestFileType.value1, 100,
                                       hostPort, persist);

        // invoke sequence generator to write 1D data to
        // file. generate sequence from 2, upto 5, with step as 2
        GeneratorServicesUtil.seqgen(2, 2, 5, seq2Ref, hostPort);
  
        // Fetch data from file to see sequence generator's output
        Data data2 = FileServiceUtil.fetchContents(seq2Ref);

        System.out.println("\nData from sequence generated for Y"); 
        FileServiceUtil.printData(data2);

        // create a 2D file to store results of xproduct.
        EndpointReference xprodRef = 
            FileServiceUtil.createFile(50, RequestFileType.value2, 100,
                                       hostPort, persist);
        
        // invoke xrpod to generate a cross product
        GeneratorServicesUtil.xprodgen(seq1Ref, seq2Ref, xprodRef, hostPort);

        // fetch the 2D data to see if cross product was generated.
        Data data3 = FileServiceUtil.fetchContents(xprodRef);

        System.out.println("\nData from cross product"); 
        FileServiceUtil.printData(data3);

        // Clean up files, delete all three files.
        FileServiceUtil.deleteFile(seq1Ref);
        FileServiceUtil.deleteFile(seq2Ref);
        FileServiceUtil.deleteFile(xprodRef);        
    }
}

