/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.tutorial.client;

import org.globus.tutorial.types.filestore.Data;

import org.globus.tutorial.types.xprod.XprodGeneratorException;
import org.globus.tutorial.types.xprod.Request;
import org.globus.tutorial.types.xprod.Response;
import org.globus.tutorial.xprod.XprodService;
import org.globus.tutorial.xprod.service.XProdServiceAddressingLocator;

import org.globus.tutorial.types.seqgen.SeqGenReq;
import org.globus.tutorial.seqgen.service.SeqGenServiceAddressingLocator;
import org.globus.tutorial.seqgen.SequenceGenerator;
import org.globus.tutorial.types.seqgen.SeqGeneratorException;

import org.apache.axis.message.addressing.EndpointReference;
import org.apache.axis.message.addressing.EndpointReferenceType;

import java.net.URL;
import javax.xml.rpc.Stub;
import javax.xml.rpc.ServiceException;

public class GeneratorServicesUtil {

    protected static final String SEQ_GEN_SERVICE
        = "/wsrf/services/SequenceGenerator";

    /**
     * Method used to generate a sequence of number within a said
     * range using the SequenceGenerator service
     * 
     * @param start 
     *        lower limit on range
     * @param step
     *        incremental step to use to generate next number
     * @param limit
     *         upper limit on range
     * @param out
     *        EPR poitning to a file created using FileStoreFactory
     * service to which the output 1D data needs to be written.
     * @param hostPort
     *        String of format host:port, host and port on which the
     *        SequenceGenerator service is running.
     */
    public static void seqgen(double start, double step, double limit, 
                             EndpointReference out, String hostPort)
        throws Exception {
        
        String err = "Error generating sequential set of 2D data\n";

        // (ADD 1) SeqGen service does not have any state, so no
        // resource. This URL is used to contact it.
        
        // (ADD 2) Construct stub to SequenceGenerator port using 
        // SeqGenServiceAddressingLocator
        
        // (ADD 3) Construct instance of SeqGenReq to invoke generate
        //  method. Refer Javadocs.

        // (ADD 4) invoke generate method. Result, which is 1D data should
        // be stored in the file specificed in file (out parameter in
        // this method.
    }

    /**
     * Method to generate a cross product of two sets of data.
     * 
     * @param seq1 
     *        EPR pointing to file with one set of data
     * @param seq2
     *        EPR pointing to file with second set of data
     * @param xprod
     *        EPR pointing to file created to store the cross product 
     * generated.
     */
    public static void xprodgen(EndpointReference seq1,
				EndpointReference seq2,
				EndpointReference xprod,
                                String machinePort) throws Exception {

        String err = "Error generating cross-product of 2 sets of 2D data\n";
        
        // (ADD 1) XProdService service does not have any state, so no
        // resource. This URL is used to contact it.

        // (ADD 2) Construct stub to XprodService port using 
        // XProdServiceAddressingLocator
        
        // (ADD 3) Constants instance of
        //  org.globus.tutorial.types.xprod.Request
        
        // (ADD 4) Invoke generate method. Result, which is 2D data s
        // written to file pointed to by xprod.
    }
}
